/***
 * GASP: a gaming services platform for mobile multiplayer games.
 * MooDS: a communication protocol used in GASP
 *
 * Copyright (C) 2004,2005 CNAM/INT - 2006 CNAM/INT/Filao
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 * 
 * Contact: gasp-team@objectweb.org
 *
 * Author: Romain Pellerin, Chanty Lim
 */
package org.mega.moods.generator;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;


import javax.xml.bind.JAXBContext;
import javax.xml.bind.JAXBException;
import javax.xml.bind.Marshaller;
import javax.xml.bind.Unmarshaller;

import org.w3._2001.xmlschema.ChoiceType;
import org.w3._2001.xmlschema.ComplexType;
import org.w3._2001.xmlschema.Element;
import org.w3._2001.xmlschema.ElementRef;
import org.w3._2001.xmlschema.Schema;
import org.w3._2001.xmlschema.ShortElementRef;

/**
 * This class parse the user XSD file of the data structure description.
 * The XSD file is written with a subset of XML Schema.
 * @author Utilisateur
 *
 */
public class TypesJAXBReader {
	private String fichierXML;
    private JAXBContext jaxbc;
    private Schema types;
    private Marshaller m;
	private Unmarshaller u;
    
	public static final int SELECT_TYPE = 1;
	public static final int UPDATE_TYPE = 2;
	public static final int INSERT_TYPE = 3;
	public static final int DELETE_TYPE = 4;
    
	/**
	 * Constructor
	 * @param fileXML the data structure description file name
	 * @throws FileNotFoundException
	 */
    public TypesJAXBReader(String fileXML) throws FileNotFoundException{ 
    	fichierXML = fileXML;
    	
    	// create a JAXB context providing xjc compiled classes use.
    	try{ 
    		jaxbc = JAXBContext.newInstance("org.w3._2001.xmlschema");
    		u = jaxbc.createUnmarshaller();
    		m = jaxbc.createMarshaller();
    		FileInputStream xml = new FileInputStream(fichierXML);
   			types = (Schema)u.unmarshal( xml );
    	}
    	catch( JAXBException je ) {
            je.printStackTrace();
		}
    }
    
    
    /**
     * Returns the mode of generator execution.
     * Mode:
     * * "normal": no compacting, manage the objects of the types declared.
     * * "compact": fusionning the types in only one class.
     * 
     * @return String - the generator execution mode 
     * @throws IOException
     */
    public String getMode() throws IOException{
        return null;//types.getMode();
    }

    
	/**
	 * Return the list of complexTypes contained by the XSD file.
	 * 
	 * @return List the types list
	 */
	public List getComplexTypes() throws IOException {			
		return types.getComplexType();
	}

	/**
	 * Return the list of elements contained by the xml schema file.
	 * 
	 * @return List the elements list
	 */
	public List getElements() throws IOException {			
		return types.getElement();
	}

	
	/**
	 * Get the root node complexTypes.
	 * @return
	 * @throws IOException
	 */
	public ComplexType getRootType() throws IOException {
		if (getRootElement().getType() != null) {
			return getComplexTypeFromName(getRootElement().getType());
		}
		/*Iterator complexTypes = getComplexTypes().iterator();
		ComplexType ct = null;
		while (complexTypes.hasNext()) {
			ct = (ComplexType)complexTypes.next();
			if ("Root".equals(ct.getName())) {
				return ct;
			}
		}*/
		return null; 
	}
	
	/**
	 * Get the root node element.
	 * @return
	 * @throws IOException
	 */
	public Element getRootElement() throws IOException {
		Iterator complexTypes = getElements().iterator();
		Element ct = null;
		while (complexTypes.hasNext()) {
			ct = (Element)complexTypes.next();
			if ("root".equals(ct.getName())) {
				return ct;
			}
		}
		return null; 
	}
	
	
	/**
	 * Get the message elements under the root node (used only in GASP mode).
	 * @return
	 * @throws IOException
	 */
	public List getMessageElements() throws IOException {
		List messageElements = new ArrayList();
		if (getRootElement() != null && getRootType().getSequence() != null) {
			Iterator complexTypes = getRootType().getSequence().getElement().iterator();
			ElementRef er = null;
			ComplexType ct = null; 
			Element elem = null; 
			while (complexTypes.hasNext()) {
				er = (ElementRef)complexTypes.next();
				elem = getElementFromName(er.getRef());
				ct = getComplexTypeFromName(elem.getType());
				if ("gmr:message".equalsIgnoreCase(er.getRef())) {
					// Add element in the Message element Choice (GASP MODE essentially)
					if (!ct.getChoice().getElement().isEmpty()) {
						messageElements.addAll(getElement(ct.getChoice()));
					} 
				} else {
					// Add element under declared under the root element
					messageElements.add(elem);
				}
			}
		}
		return messageElements; 
	}
	
	/**
	 * Get the referenced element from the complex type.
	 * @return
	 * @throws IOException
	 */
	public ElementRef getReferenceElementFromComplexType(ComplexType complexType) throws IOException {
		if (getRootElement() != null && getRootType().getSequence() != null) {
			Iterator complexTypes = getRootType().getSequence().getElement().iterator();
			ElementRef er = null;
			ComplexType ct = null; 
			Element elem = null; 
			while (complexTypes.hasNext()) {
				er = (ElementRef)complexTypes.next();
				elem = getElementFromName(er.getRef());
				ct = getComplexTypeFromName(elem.getType());
				if (complexType.getName().equalsIgnoreCase(ct.getName())) {
					return er;
				}
			}
		}
		return null; 
	}

	/**
	 * Get the referenced element name from the given complexType.
	 * @param complexType from where the referenced must be found.
	 * @return the referenced element name from the given complexType.
	 * @throws IOException
	 */
	public String getReferenceElementNameFromComplexType(ComplexType complexType) throws IOException {
		ElementRef er = getReferenceElementFromComplexType(complexType);
		int dotIndex = 0;
		if (er.getRef().indexOf(":") != -1) {
			dotIndex = er.getRef().indexOf(":");
		}
		return er.getRef().substring(dotIndex+1, er.getRef().length());
	}
	
	/**
	 * Get the ComplexType for the given ChoiceType parameter.
	 * @param choice
	 * @return
	 * @throws IOException
	 */
	public List getComplexType(ChoiceType choice) throws IOException {
		List elements = new ArrayList();
		Iterator choices = choice.getElement().iterator();
		ShortElementRef e = null;
		String elementName = null;
		ComplexType complexType = null;
		while (choices.hasNext()) {
			e = (ShortElementRef)choices.next();
			elementName = e.getRef();
			complexType = getComplexTypeFromName(elementName);
			if (complexType != null) {				
				elements.add(complexType);
			}
		}
		return elements; 
	}	
	
	/**
	 * Get the Element for the given Choice parameter.
	 * @param choice
	 * @return
	 * @throws IOException
	 */
	public List getElement(ChoiceType choice) throws IOException {
		List elements = new ArrayList();
		Iterator choices = choice.getElement().iterator();
		ShortElementRef e = null;
		String elementName = null;
		Element elem = null;
		while (choices.hasNext()) {
			e = (ShortElementRef)choices.next();
			elementName = e.getRef();
			elem = getElementFromName(elementName);
			if (elem != null) {				
				elements.add(elem);
			}
		}
		return elements; 
	}	
	
	/**
	 * Get the ComplexType from the given name.
	 * @param name
	 * @return
	 * @throws IOException
	 */
	public ComplexType getComplexTypeFromName(String name) throws IOException {
		Iterator complexTypes = getComplexTypes().iterator();
		ComplexType ct = null;
		while (complexTypes.hasNext()) {
			ct = (ComplexType)complexTypes.next();
			// COmpare the gmr:newPlayer without the suffix "gmr:"
			if (name.substring(name.indexOf(":")+1,name.length()).equalsIgnoreCase(ct.getName())) {				
				return ct; 
			}
		}
		return null; 
	}
	
	/**
	 * Get the Element from the given name.
	 * @param name
	 * @return
	 * @throws IOException
	 */
	public Element getElementFromName(String name) throws IOException {
		Iterator complexTypes = getElements().iterator();
		Element ct = null;
		while (complexTypes.hasNext()) {
			ct = (Element)complexTypes.next();
			// COmpare the gmr:newPlayer without the suffix "gmr:"
			if (name.substring(name.indexOf(":")+1,name.length()).equalsIgnoreCase(ct.getName())) {				
				return ct; 
			}
		}
		return null; 
	}
	
	public String getOnElementStartFromType(ComplexType type) throws IOException {
		if (type != null && type.getAnnotation() != null && 
				type.getAnnotation().getAppinfo() != null && 
				type.getAnnotation().getAppinfo().getEvents() != null && 
				type.getAnnotation().getAppinfo().getEvents().getOnElementStart() != null) {
			return type.getAnnotation().getAppinfo().getEvents().getOnElementStart().getValue().trim();
		} else {
			return null;
		}
	}
	
	public String getOnElementStartFromElement(Element elem) throws IOException {
		ComplexType type = getComplexTypeFromName(elem.getType());
		if (type != null && type.getAnnotation() != null && 
				type.getAnnotation().getAppinfo() != null && 
				type.getAnnotation().getAppinfo().getEvents() != null && 
				type.getAnnotation().getAppinfo().getEvents().getOnElementStart() != null) {
			return type.getAnnotation().getAppinfo().getEvents().getOnElementStart().getValue().trim();
		} else {
			return null;
		}
	}
	
	public String getOnElementStartFromName(String name) throws IOException {
		Element elem = getElementFromName(name);
		if (elem != null) {
			return getOnElementStartFromElement(elem);
		} else {
			return null;
		}
	}
	
	public String getRequestFromType(ComplexType type, int requestType) throws IOException {
		if (type.getAnnotation().getAppinfo() != null &&
				type.getAnnotation().getAppinfo().getRequests() != null) { 
			switch (requestType) {
				case SELECT_TYPE : 
					if (type.getAnnotation().getAppinfo().getRequests().getSelect() != null) 
						return type.getAnnotation().getAppinfo().getRequests().getSelect().getValue().trim();
				case UPDATE_TYPE : 
					if (type.getAnnotation().getAppinfo().getRequests().getUpdate() != null) 
						return type.getAnnotation().getAppinfo().getRequests().getUpdate().getValue().trim();
				case INSERT_TYPE : 
					if (type.getAnnotation().getAppinfo().getRequests().getInsert() != null)
						return type.getAnnotation().getAppinfo().getRequests().getInsert().getValue().trim();
				case DELETE_TYPE : 
					if (type.getAnnotation().getAppinfo().getRequests().getDelete() != null) 
						return type.getAnnotation().getAppinfo().getRequests().getDelete().getValue().trim();
	            default :
	            	if (type.getAnnotation().getAppinfo().getRequests().getSelect() != null)
	            		return type.getAnnotation().getAppinfo().getRequests().getSelect().getValue().trim();
			}
		}
		return null;
	}
	
	public String getRequestFromName(String name, int requestType) throws IOException {
		return getRequestFromType(getComplexTypeFromName(name), requestType);
	}
	
	public boolean isRequestPresent(ComplexType type, int requestType) {
		if (type.getAnnotation() != null && type.getAnnotation().getAppinfo() != null &&
				type.getAnnotation().getAppinfo().getRequests() != null) { 
			switch (requestType) {
				case SELECT_TYPE : 
					if (type.getAnnotation().getAppinfo().getRequests().getSelect() != null) 
						return true;
				case UPDATE_TYPE : 
					if (type.getAnnotation().getAppinfo().getRequests().getUpdate() != null) 
						return true;
				case INSERT_TYPE : 
					if (type.getAnnotation().getAppinfo().getRequests().getInsert() != null)
						return true;
				case DELETE_TYPE : 
					if (type.getAnnotation().getAppinfo().getRequests().getDelete() != null) 
						return true;
			}
		}
		return false;
	}
	
	public boolean isRequestPresent(int requestType) throws IOException {
		Iterator complexTypes = getComplexTypes().iterator();
		ComplexType ct = null;
		while (complexTypes.hasNext()) {
			ct = (ComplexType)complexTypes.next();
			boolean isRequestPresent = isRequestPresent(ct, requestType);
			if (isRequestPresent) {
				return isRequestPresent;
			}
		}
		return false;		
	}	
}

